/*
 * Dwarf - ATL H323 client wrapper to openH323
 * Original version by Damian Slee
 * 
 * Large portions of this were taken from the openH323 voxilla example.
 *
 *
 * The contents of this file are subject to the Mozilla Public License
 * Version 1.0 (the "License"); you may not use this file except in
 * compliance with the License. You may obtain a copy of the License at
 * http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an "AS IS"
 * basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See
 * the License for the specific language governing rights and limitations
 * under the License.
 *
 * The Original Code is Portable Windows Library.
 *
 * The Initial Developer of the Original Code is Equivalence Pty. Ltd.
 *
 * Portions are Copyright (C) 1993 Free Software Foundation, Inc.
 * All Rights Reserved.
 *
 * Contributor(s): ______________________________________.
 *
 *
 */


#include "dwarf.h"
#include "version.h"
#include "atlh323.h"
#include "gsmcodec.h"



///////////////////////////////////////////////////////////////
/* We have to have a PProcess running for the life of the DLL to give
 * openh323 a static instance of PProcess to get system information.
 * This class is defined with PDECLARE_PROCESS().  See pprocess.h from pwlib.
 *
 */
PDECLARE_PROCESS(Dwarf,PProcess,"Dwarf","vbH323",MAJOR_VERSION,MINOR_VERSION,BUILD_TYPE,BUILD_NUMBER)
static Dwarf RedDwarf;  // active for the life of the DLL
void Dwarf::Main()  // implement the virtual function which is never called
{
}

///////////////////////////////////////////////////////////////

MyH323EndPoint::MyH323EndPoint(VBFire *_vbEvents)
{
	vbEvents = _vbEvents;
	silenceOn=TRUE;
	noFastStart=FALSE;
}



BOOL MyH323EndPoint::Initialise()
{
  noFastStart = FALSE;
  InitialiseCodecs();
  return TRUE;
}


/*
 * This may be moved around in the future 
 */

BOOL MyH323EndPoint::InitialiseCodecs()
{
  H323Capability * gsm    = NULL;
  H323Capability * ulaw64 = NULL;
  H323Capability * alaw64 = NULL;

  gsm    = new H323_GSM0610Capability;
//  if (args.HasOption("gsm"))  // set default
//      AddCapability(gsm);

  ulaw64 = new H323_G711Capability(H323_G711Capability::muLaw, H323_G711Capability::At64k);
//  if (args.HasOption("g711-ulaw")) // set default
//      AddCapability(ulaw64);

  alaw64 = new H323_G711Capability(H323_G711Capability::ALaw, H323_G711Capability::At64k);
//  if (args.HasOption("g711-Alaw")) // set default
//    AddCapability(alaw64);

  if (gsm != NULL)
    SetCapability(0, 0, gsm);

  if (ulaw64 != NULL)
    SetCapability(0, 0, ulaw64);

  if (alaw64 != NULL)
    SetCapability(0, 0, alaw64);

  return TRUE;
}



H323Connection * MyH323EndPoint::CreateConnection(unsigned callReference)
{
	/* Spawn a new connection */
  return new MyH323Connection(*this, callReference);
}


void MyH323EndPoint::OnConnectionEstablished(H323Connection & connection,
                                             const PString & token)
{
PString p;

	currentCallToken = token;
	// Send Event to VB
	p  = connection.GetRemotePartyName();
	vbEvents->fire_Connected(vbEvents->a2BSTR(p));
}


void MyH323EndPoint::OnConnectionCleared(H323Connection & connection, const PString &)
{
PString p;

	currentCallToken = "";
	// Send Event to VB
	p = connection.GetRemotePartyName();
	vbEvents->fire_Disconnected(vbEvents->a2BSTR(p));
}



BOOL MyH323EndPoint::OpenAudioChannel(H323Connection & connection,
                                         BOOL isEncoding,
                                         unsigned bufferSize,
                                         H323AudioCodec & codec)
{
  codec.EnableSilenceDetection(silenceOn);
  return H323EndPoint::OpenAudioChannel(connection, isEncoding, bufferSize, codec);
}


BOOL MyH323EndPoint::OpenVideoDevice(H323Connection & connection,
                                     BOOL isEncoding,
                                     H323VideoCodec & codec)
{
//  if (isEncoding || h261VideoDevice.IsEmpty())
//    return H323EndPoint::OpenVideoDevice(connection, isEncoding, codec);

  // Dump received video to a file.
//  if (h261VideoDevice *= "ppm")
//    return codec.AttachDevice(new PPMVideoOutputDevice());

  return FALSE;
}


void MyH323EndPoint::ListenForIncomingCalls()
{
	H323ListenerTCP * listener = new H323ListenerTCP(*this);

    if (!StartListener(listener)) 
	{		
		// Send Event to VB
		vbEvents->fire_Error(errFailToListen,OLESTR("Could not open H.323 listener port"));
      	return;
    }
    answerState = uiWaitingForCall;
}


void MyH323EndPoint::MakeOutgoingCall(const PString & dest, const PString & proxy)
{
  H323TransportTCP * transport = new H323TransportTCP(*this);

  if (!proxy) {
    if (!transport->SetRemoteAddress("ip:"+proxy)) 
	{
		vbEvents->fire_Error(errInvalidDest,OLESTR("Invalid gateway address"));
        return;
    }
  }
  else
  {
    if (!transport->SetRemoteAddress("ip:"+dest)) {
		// Send Event to VB
		vbEvents->fire_Error(errInvalidDest,OLESTR("Invalid destination address"));
		return;
    }
  }

  if (!MakeCall(dest, transport, currentCallToken)) 
  {	
		// Send Event to VB		
		vbEvents->fire_Error(errFailToCall,OLESTR("Failed to call destination"));
		return;
  }
}


void MyH323EndPoint::AnswerCall(H323Connection::AnswerCallResponse response)
{
	// Has VB user called us back while in the event called from MyConnection::OnAnswerCall?
	//  If so, set a flag to tell it to accept the call in OnAnswerCall(), rather than here.
	if (answerState == uiAnsweringCallVB)
	{
		// accepted the call
		if (response == H323Connection::AnswerCallNow)
			answerState = uiAnsweredCall;
		else // denied call
			answerState = uiWaitingForCall;
		return;
	}

    H323Connection * connection = FindConnectionWithLock(currentCallToken);
    if (connection == NULL)
        return;

    connection->AnsweringCall(response);
    connection->Unlock();
}



void MyH323EndPoint::Hangup()
{
	if (!currentCallToken) 
	{
		if (!ClearCall(currentCallToken))
			vbEvents->fire_Error(errFailToCall,OLESTR("Could not hang up current call"));
    }
}


void MyH323EndPoint::SendUserInput(PString str)
{
	H323Connection * connection = FindConnectionWithLock(currentCallToken);

    if (connection != NULL)
	{
        connection->SendUserInput(str);
        connection->Unlock();
	}
}


void MyH323EndPoint::RegisterGatekeeper(const PString & gkName)
{
    if (SetGatekeeper("ip:"+gkName, new H323TransportUDP(*this)))
		vbEvents->fire_Error(errFailToRegisterGatekeeper,OLESTR("Failed to register with Gatekeeper"));
}

///////////////////////////////////////////////////////////////

MyH323Connection::MyH323Connection(MyH323EndPoint & ep, unsigned callReference)
  : H323Connection(ep, callReference),
    myEndpoint(ep)
{
}


BOOL MyH323Connection::OnIncomingCall(const H323SignalPDU &, H323SignalPDU &)
{
  if (!myEndpoint.currentCallToken) 
  {
    //cout << "Incoming call rejected, line busy!" << endl;
    return FALSE;
  }

  return TRUE;
}


H323Connection::AnswerCallResponse
     MyH323Connection::OnAnswerCall(const PString & caller,
                                    const H323SignalPDU & /*setupPDU*/,
                                    H323SignalPDU & /*connectPDU*/)
{
    if (myEndpoint.autoAnswer)
	{
         return AnswerCallNow;
	}

    myEndpoint.currentCallToken = GetCallToken();
    myEndpoint.answerState = MyH323EndPoint::uiAnsweringCallVB;

	// Send Event to VB
	myEndpoint.vbEvents->fire_ReceivedCall(myEndpoint.vbEvents->a2BSTR(caller));

	// Did VB user respond in the above event and call MyEndPoint::AnswerCall()?
	switch (myEndpoint.answerState)
	{
	case MyH323EndPoint::uiAnsweredCall: // yes, answer the call now
        return AnswerCallNow;
		break;
	case MyH323EndPoint::uiWaitingForCall: // no, denied the call
        return AnswerCallDenied;
		break;
	default:                               // didn't do anything.  pending
		// VB has signaled what to do yet, so wait.
		myEndpoint.answerState = MyH323EndPoint::uiAnsweringCall;
		return AnswerCallPending;
	}
}




void MyH323Connection::OnSelectLogicalChannels()
{
  if (fastStartState != FastStartDisabled && myEndpoint.noFastStart)
    return;

#if 1 // Clear to temporarily disable the transmitter start up
  H323Connection::OnSelectLogicalChannels();
#endif
}


BOOL MyH323Connection::OnOpenLogicalChannel(const H245_OpenLogicalChannel & openPDU,
                                            H245_OpenLogicalChannelAck & ackPDU,
                                            unsigned & errorCode)
{
#if 0 // Set to temporarily disable the receiver acceptance
  return FALSE;
#else
  return H323Connection::OnOpenLogicalChannel(openPDU, ackPDU, errorCode);
#endif
}


BOOL MyH323Connection::OnStartLogicalChannel(H323Channel & channel)
{
  if (!H323Connection::OnStartLogicalChannel(channel))
    return FALSE;

 // report codec selection here
  switch (channel.GetDirection()) {
    case H323Channel::IsTransmitter :
      myEndpoint.TxCodec = channel.GetCapability().GetCapabilityNumber();
      break;

    case H323Channel::IsReceiver :
      myEndpoint.RxCodec = channel.GetCapability().GetCapabilityNumber();
      break;

    default :
      break;
  }
  return TRUE;
}


BOOL MyH323Connection::OnAlerting(const H323SignalPDU & /*alertingPDU*/,
                                  const PString & username)
{
	myEndpoint.vbEvents->fire_RemoteRinging();
	return TRUE;
}


void MyH323Connection::OnUserInputString(const PString & value)
{
	myEndpoint.vbEvents->fire_RemoteUserInput(myEndpoint.vbEvents->a2BSTR(value));
}



