#ifndef DWARF_H
#define DWARF_H
/*
 * Dwarf - ATL H323 client wrapper to openH323
 * Original version by Damian Slee
 * 
 * Large portions of this were taken from the openH323 voxilla example.
 *
 *
 * The contents of this file are subject to the Mozilla Public License
 * Version 1.0 (the "License"); you may not use this file except in
 * compliance with the License. You may obtain a copy of the License at
 * http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an "AS IS"
 * basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See
 * the License for the specific language governing rights and limitations
 * under the License.
 *
 * The Original Code is Portable Windows Library.
 *
 * The Initial Developer of the Original Code is Equivalence Pty. Ltd.
 *
 * Portions are Copyright (C) 1993 Free Software Foundation, Inc.
 * All Rights Reserved.
 *
 * Contributor(s): ______________________________________.
 *
 *
 */

#include <ptlib.h>
#include "h323.h"

#include "fire.h"



class MyH323EndPoint : public H323EndPoint
{
    PCLASSINFO(MyH323EndPoint, H323EndPoint);
  public:
	MyH323EndPoint(VBFire*);

    // overrides from H323EndPoint
    virtual H323Connection * CreateConnection(unsigned callReference);
    virtual void OnConnectionEstablished(H323Connection &, const PString &);
    virtual void OnConnectionCleared(H323Connection &, const PString &);
    virtual BOOL OpenAudioChannel(H323Connection &, BOOL, unsigned, H323AudioCodec &);
    virtual BOOL OpenVideoDevice(H323Connection &, BOOL, H323VideoCodec &);


    // new functions
    BOOL Initialise();
    virtual BOOL InitialiseCodecs();
    virtual void ListenForIncomingCalls();
    virtual void MakeOutgoingCall(const PString &,const PString &);
	virtual void AnswerCall(H323Connection::AnswerCallResponse response);
	virtual void Hangup();
	virtual void RegisterGatekeeper(const PString &);
	virtual void SendUserInput(PString str);

	PString currentCallToken;
	BOOL noFastStart;
	BOOL autoAnswer;
    BOOL   silenceOn;
	long RxCodec;
	long TxCodec;

  protected:
    // only used internally
	VBFire *vbEvents;

	// Monitor the states that we are in
    enum 
	{
      uiWaitingForCall,
	  uiAnsweringCallVB,
      uiAnsweringCall,
      uiAnsweredCall,
    } answerState;

	friend class MyH323Connection;
};



class MyH323Connection : public H323Connection
{
    PCLASSINFO(MyH323Connection, H323Connection);

  public:
    MyH323Connection(MyH323EndPoint &, unsigned);

    // overrides from H323Connection
    BOOL OnIncomingCall(const H323SignalPDU &, H323SignalPDU &);
    AnswerCallResponse OnAnswerCall(const PString &, const H323SignalPDU &, H323SignalPDU &);
    void OnSelectLogicalChannels();
    BOOL OnOpenLogicalChannel(const H245_OpenLogicalChannel &, H245_OpenLogicalChannelAck &, unsigned &);
    BOOL OnStartLogicalChannel(H323Channel &);
    BOOL OnAlerting(const H323SignalPDU &, const PString &);
    void OnUserInputString(const PString &);

  protected:
    MyH323EndPoint & myEndpoint;
};



#endif